
#include "math.h"

#include <packages.h>

#include "proFit_definitions.h"
/**************************  functions called by proFit *******************************/
/****** the definitions for the following functions must be provided by you  **********/

pascal void func (double* x, paramArray a, double* y,globalsPtrT InfPtr);

pascal void derivatives(double* x, paramArray a, paramArray dyda,globalsPtrT InfPtr);

pascal char check(short ParamNr, paramArray a, paramModesArray mode, paramNamesArray aName, globalsPtrT InfPtr);

pascal void first (paramArray a,globalsPtrT InfPtr);

pascal void freeUserMemory (globalsPtrT InfPtr );
	
pascal void  main (	str20 name,	//the name of the function }
					descrType descr,	// a description of the function for the parameters window
					pascal void (**firstPtr)(paramArray, globalsPtrT),
								// pointer to the procedure 'first' or nil if not used
					pascal char (**checkPtr)(short, paramArray, paramModesArray, paramNamesArray, globalsPtrT) ,
								// pointer to the procedure 'check' or nil if not used
					pascal void (**funcPtr)(double*, paramArray, double*, globalsPtrT) ,
					 			// pointer to the procedure 'func' or nil if not used
					pascal void (**derivativesPtr)(double*, paramArray, paramArray, globalsPtrT) ,
					 			// pointer to the procedure 'derivatives' or nil if not used }
					pascal void (**freeUserMPtr)(globalsPtrT) ,
					  			// pointer to the procedure 'FreeUserMemory' or nil if not used
					short* nrParams,			// the number of parameters the function will use (  maxNrParams ) }
					paramArray a,			// the default values  for the parameters
					paramModesArray mode,	// the default  modes for the parameters
					paramNamesArray aName, 	// the parameter names
					globalsPtrT InfPtr);	// pointer to a struct containing some proFit globals available to you
/*******************************************************************************/

typedef struct {		
				double  xx[15],yy[15];
				short oldindex;
				short pts,Npts;										
																
		} MyGlobals,**HandToMyGlobals;
/*
this works in the same way as usual variable declarations.The Handle
InfPtr->TheHandle must be initialized and set to point to this struct
*/

/*
To access comfortably the variables in this record we defined the type 'HandToMyGlobals'.
If H is of the type HandToMyGlobals and H == InfPtr->TheHandle we can access the variables
using (*H)->a3 etc. See the example in the procedure func.
*/



#include "mathematics.h"


static double getAverage(double* a,short elems)
{double sum=0;
 short i=0;
 for (i=0;i<elems;i++) sum += a[i];
 return(sum/elems);
}


//*************************
//***	    first		***
//*************************
// this is the function called by proFit before any series of calls to func


static double extrapolate(double x, double x1, double y1, double x2, double y2)
{
return y1 + (y2 - y1) * (x - x1) / (x2 -x1);
}

pascal void first (paramArray a,globalsPtrT InfPtr)
{
	double* *xx,*yy;
	short i,Npts=a[0];

	HandToMyGlobals H = (HandToMyGlobals) InfPtr->theHandle;
		
	for (i=2;i<2*Npts+1;i+=2) {if (a[i+2]<a[i]) a[i+2]=a[i];}
	
	for (i=2;i<2*Npts+2;i+=2) (**H).xx[i/2]=a[i];
	for (i=3;i<2*Npts+2;i+=2) (**H).yy[(i-1)/2]=a[i];
}



//*************************
//***		func		***
//*************************
// this is the function used by proFit when fitting, plotting, etc .. 
void pascal func (double* x, paramArray a, double* y,globalsPtrT InfPtr)
{	short Npts=a[0];
	HandToMyGlobals H = (HandToMyGlobals) InfPtr->theHandle;
	if (Npts>2) {
		short index=(**H).oldindex;
		double *px,*py;
		HLock(H);
		px=(**H).xx;py=(**H).yy;
		hunt(px,Npts,*x,&index);
	
		if (index==0) { *y=extrapolate(*x,px[1],py[1],px[2],py[2]);return;}
		if (index==Npts) {*y=extrapolate(*x,px[Npts-1],py[Npts-1],px[Npts],py[Npts]);return;}
	
		if (a[1]==0) *y=py[index] + (py[index+1] - py[index]) * (*x - px[index])/(px[index+1] - px[index]);
		else
		{	double oldx1=px[1],oldy1=py[1],oldxN=px[Npts],oldyN=py[Npts];
			double pox,poy,p1x,p1y,p2x,p2y;
			px[1]-=px[2]-px[1];py[1]-=py[2]-py[1];
			px[Npts]+=px[Npts]-px[Npts-1];py[Npts]+=py[Npts]-py[Npts-1];
						
			if (*x < (px[index]+px[index+1])/2)
				{	pox=(px[index-1]+px[index])/2;poy=(py[index-1]+py[index])/2;
					p1x=px[index];p1y=py[index];
					p2x=(px[index]+px[index+1])/2;p2y=(py[index]+py[index+1])/2;
				}
			else
				{
					pox=(px[index]+px[index+1])/2;poy=(py[index]+py[index+1])/2;
					p1x=px[index+1];p1y=py[index+1];
					p2x=(px[index+1]+px[index+2])/2;p2y=(py[index+1]+py[index+2])/2;
				}
			Bezier(pox,poy,p1x,p1y,p2x,p2y,*x,y);
			px[1]=oldx1;py[1]=oldy1;
			px[Npts]=oldxN;py[Npts]=oldyN;
		}
		(**H).oldindex=index;
		HUnlock(H);
	}
}

//*************************
//***	 derivatives	***
//*************************
// this is the function used by proFit when fitting
 
pascal void derivatives(double* x, paramArray a, paramArray dyda,globalsPtrT InfPtr)
	{
//we pass nil for the derivatives procedure in main }
	}

//*************************
//***	    check		***
//*************************
// this is the function called by proFit when it changes one of the parameters

pascal char check(short ParamNr, paramArray a, paramModesArray mode, paramNamesArray aName, globalsPtrT InfPtr)
	{	short grad,i,NrPoints;
	
		if (ParamNr == 1) {
			if ((1 < (a)[0]) && ((a)[0] <= (maxNrParams - 2)/2))
				{	
					grad = (a)[0]+0.5;
					(a)[0] = grad;
					NrPoints = 2*grad + 2;
					for (i = 2;i< NrPoints;i++)
						(mode)[i] = active;
					for	(i = NrPoints;i< maxNrParams;i++)
						(mode)[i] = inactive;
					{
					short xCol=xColumn(InfPtr->proFitInfo), yCol=yColumn(InfPtr->proFitInfo);
					if (xCol && yCol)
					{	short acount;
						double NrDataPoints,NrDataPoints2,count=0;
//InfPtr->proFitInfo->nrRows=500;
						for (i=1;i<=InfPtr->proFitInfo->nrRows;i++) {
							if (InfPtr->proFitInfo->testData(xCol,i) && InfPtr->proFitInfo->testData(yCol,i)) {
								count +=1;
							}
						}
						NrDataPoints=1;NrDataPoints2=(count-1)/(grad-1);count=1;acount=1;
						for (i=1;i<=InfPtr->proFitInfo->nrRows;i++) {
							if (InfPtr->proFitInfo->testData(xCol,i) && InfPtr->proFitInfo->testData(yCol,i)) {
								if ((fabs(count-NrDataPoints)<1) && (acount<=grad)) {
									NrDataPoints=1.0+NrDataPoints2*acount;
									a[2*acount]=InfPtr->proFitInfo->getData(xCol,i);
									a[2*acount+1]=InfPtr->proFitInfo->getData(yCol,i);
									acount++;
								}
							count +=1;
							}
						}
					}
					}
					return update;
				}
			else return bad;
		}
		return ok_;
	}


//*************************
//***  freeUserMemory	***
//*************************
// this is the function called by proFit when your func is removed from the func menu

pascal void freeUserMemory (globalsPtrT InfPtr )
	{
		if (InfPtr->theHandle !=0) DisposHandle(InfPtr->theHandle);
	}
	



//*************************
//***	    main		***
//*************************
// this is the function called by proFit when your func is first inserted in the func menu

pascal void  main 
									(str20 name,					//the name of the function }
									descrType descr,			// a description of the function for the parameters window }
									pascal void (**firstPtr)(paramArray, globalsPtrT),
														// pointer to the procedure 'first' or nil if not used }
									pascal char (**checkPtr)(short, paramArray, paramModesArray, paramNamesArray, globalsPtrT) ,
									 					// pointer to the procedure 'check' or nil if not used }
									pascal void (**funcPtr)(double*, paramArray, double*, globalsPtrT) ,
									 					// pointer to the procedure 'func' or nil if not used }
									pascal void (**derivativesPtr)(double*, paramArray, paramArray, globalsPtrT) ,
									 			// pointer to the procedure 'derivatives' or nil if not used }
									pascal void (**freeUserMPtr)(globalsPtrT) ,
									  			// pointer to the procedure 'FreeUserMemory' or nil if not used }
									short* nrParams,			// the number of parameters the function will use (  maxNrParams ) }
									paramArray a,			// the default values  for the parameters }
									paramModesArray mode,	// the default  modes for the parameters }
									paramNamesArray aName, 		// the parameter names }
									globalsPtrT InfPtr)				// pointer to a record containing global variables and some proFit globals available to the user }
{ 	short i;
	InfPtr->theHandle = NewHandle(sizeof(MyGlobals));
/*this allocates space for the variables declared in the record 'MyGlobals' 
the function 'sizeof' calculates the memory space that your record needs
just add new variable names to the record MyGlobals if you need more global variables*/

		if ((InfPtr->theHandle == nil) || (MemError() != noErr)) 
		SysBeep(1);
// handle error, if an error occured you are not allowed to use the Handle InfPtr^.theHandle}
		else
		{ 	HandToMyGlobals H=(HandToMyGlobals) InfPtr->theHandle;
			(**H).oldindex=1;
		};

		CopyToPascalStr("Bzier",name,20); //name of our function

		
		CopyToPascalStr("y = smooth curve described by the Npts points (xi,yi);", descr[0],80);
		CopyToPascalStr("smth=0: line connecting the points, smth=1: Bzier Curve", descr[1],80);

		*firstPtr = &first; //if it were implemented, we would have written firstPtr = &first;}
		*checkPtr = &check;
		*funcPtr = &func;
		*freeUserMPtr = &freeUserMemory;
		*derivativesPtr = 0;//we didn't implement this function}
		*nrParams = 30;	//Number of parameters used. This MUST correspond to the number
					 	// of parameters you are using
		a[0] = 14;
		mode[0] = inactive;
		CopyToPascalStr("Npts",aName[0],maxParamNameLength);	
		a[1] = 1;
		mode[1] = inactive;
		CopyToPascalStr("smth", aName[1],maxParamNameLength);	
		{Str255 ss;short cc=0;
		for (i=2;i<*nrParams;i++) {a[i]=i;mode[i] = active;}
		for (i=2;i<*nrParams;i+=2)
				{NumToString(i/2,ss);ss[ss[0]+1]=0;ss[0]='x';
				CopyToPascalStr((char*) ss, aName[i],maxParamNameLength);
				a[i]=i/2;}
		for (i=3;i<*nrParams;i+=2)
				{NumToString((i-1)/2,ss);ss[ss[0]+1]=0;ss[0]='y';
				CopyToPascalStr((char*) ss, aName[i],maxParamNameLength);
				a[i]=cc;cc=(!cc);}
		}
}  // main }
	
