/******************************************************************************************/
/*
This is an example of a C definition of a function to be installed in proFit's function menu.
This C source can be compiled by many C compilers for the macintosh with only minor changes.
This C source file uses another file, called codeGlobals, that contains all the constants and typedefs definitions
needed.
see the proFit user manual for further details

 To add this function to proFit's func menu, proceed as follows:									

    compile this unit with your compiler  tell your compiler to save it as a CODE resource. 		
    (Most current compilers on the Macintosh (MPW, Think C, ...) support code resources 
    this feature is used for the compilation of device drivers, desk accessories,
    window definition procedures etc. Find more details in your compiler's manual.)
    The code resource must have type 'fnrf' or 'fnrn' depending on whether you generate
	code for the floating point unit (coprocessor) or not ! Its ID is not important but should
	be larger than 128. 									
    Save this resource and add it to proFit using the External Modules...   commmand.				

 IMPORTANT NOTE 1: 
 The special way in which global variables are treated in Macintosh compilers makes it 			
 impossible to use global variables in spearated code segments. This means that you  			
 MUST NOT define any varibales outside your procedures! The example program 				
 below shows you how you can store global data in the userglobals record.
 
 IMPORTANT NOTE 2:
 proFit uses pascal calling conventions to call your procedure and funtions.
 Therefore every function that is called by proFit is preceded by the keyword
 pascal in the function definitions below.			
 		
 VERY IMPORTANT NOTE:
 You MUST use the following declaration of the procedure main EXACTLY as it is given here.
 ANY change is guaranteed to cause very ugly crashes !!!		
*/									
/******************************************************************************************/

#include <math.h>

#include "proFit_definitions.h"

/**************************  functions called by proFit *******************************/
/****** the definitions for the following functions must be provided by you  **********/

pascal void func (double* x, paramArray a, double* y,globalsPtrT InfPtr);

pascal void derivatives(double* x, paramArray a, paramArray dyda,globalsPtrT InfPtr);

pascal char check(short ParamNr, paramArray a, paramModesArray mode, paramNamesArray aName, globalsPtrT InfPtr);

pascal void first (paramArray a,globalsPtrT InfPtr);

pascal void freeUserMemory (globalsPtrT InfPtr );
	
pascal void  main (	str20 name,	//the name of the function }
					descrType descr,	// a description of the function for the parameters window
					pascal void (**firstPtr)(paramArray, globalsPtrT),
								// pointer to the procedure 'first' or nil if not used
					pascal char (**checkPtr)(short, paramArray, paramModesArray, paramNamesArray, globalsPtrT) ,
								// pointer to the procedure 'check' or nil if not used
					pascal void (**funcPtr)(double*, paramArray, double*, globalsPtrT) ,
					 			// pointer to the procedure 'func' or nil if not used
					pascal void (**derivativesPtr)(double*, paramArray, paramArray, globalsPtrT) ,
					 			// pointer to the procedure 'derivatives' or nil if not used }
					pascal void (**freeUserMPtr)(globalsPtrT) ,
					  			// pointer to the procedure 'FreeUserMemory' or nil if not used
					short* nrParams,			// the number of parameters the function will use (  maxNrParams ) }
					paramArray a,			// the default values  for the parameters
					paramModesArray mode,	// the default  modes for the parameters
					paramNamesArray aName, // the parameter names
					globalsPtrT InfPtr);	// pointer to a struct containing some proFit globals available to you
/***************************************************************************************/



/*
This is a description of the method you can use to store global data for your function.

proFit provides a pointer parameter to everyone of your functions. This parameter
points to a struct containing some proFit info (please check the proFit user manual)
and a Handle that you can initialise and use as you wish.
if you want to use your own global variables you can declare a struct type similar to
the following containing all the variables you need. You then set the Handle 'TheHandle'
in the UserglobalsRec to point to this struct of yours. Before doing this you must
allocate place for your data structure in memory by calling the function
'NewHandle(size_of_your_data)' in your procedure main.			
*/	
		typedef struct {			
				double a1, a2, a3;
				double V1[500], V2[500], V3[500];
			} MyGlobals,**HandToMyGlobals;
/*
this works in the same way as usual variable declarations.The handle
InfPtr->TheHandle must be initialized and set to point to this struct

To access the variables in this record, we defined the type 'HandToMyGlobals'.
If H is of the type HandToMyGlobals and H == InfPtr->TheHandle we can access the variables
using (*H)->a3 etc. See the example in the procedure func.
*/

//*************************
//***		func		***
//*************************
// this is the function used by proFit for fitting, plotting, etc .. 

void pascal func (double* x, paramArray a, double* y,globalsPtrT InfPtr)
	{	const double pp = 0.2316419;
		const double pi = 3.1415926535897932385;

		HandToMyGlobals H ; // it is more comfortable to use H instead of InfPtr->theHandle
		double t, x1, sum, buf;
		int i ;
	
		H = (HandToMyGlobals) InfPtr->theHandle;
		(**H).a3=4;
/*
if you have a record of user globals you can now access it using  the local variable H
i.e 'something = somethingelse+ (*H)->a3;' this example function does not use this facility.
The above is just an example of the mechanism.
*/


//The following code calculates the Error Function using an approximate formula

		x1 = (*x - a[0]) / a[1]; 			// a[0] and a[1] are the mean value and the standard deviation
		t = 1 / (1 + pp * fabs(x1));
		sum = 0;
		buf = t;
		for (i = 1;i< 6;i++)
			{
				sum += InfPtr->V[i] * buf; 	// the coefficients were stored in the global array V
				buf *= t;			 		// in the 'main' procedure
			};
		t = exp(-x1*x1/ 2.0)/ sqrt(2 * pi) * sum;
		if (x1 >= 0 )
			*y = a[3] * (1 - t + a[2]);
		else
			*y = a[3] * (t + a[2]);   //a[2] is just an additive constant 
	}

//*************************
//***	 derivatives	***
//*************************
// this is the function used by proFit when fitting
 
pascal void derivatives(double* x, paramArray a, paramArray dyda,globalsPtrT InfPtr)
	{
//we do not calculate any derivatives. Therefore we pass nil for the derivatives procedure in main 
	}

//*************************
//***	    check		***
//*************************
// this is the function called by proFit when it changes one of the parameters

pascal char check(short ParamNr, paramArray a, paramModesArray mode, paramNamesArray aName, globalsPtrT InfPtr)
	{
		if ( ((ParamNr = 2) && (a[ParamNr-1] <= 0))
		 || ((ParamNr = 4) && (a[ParamNr-1] == 0)) )
			return bad;
		else
			return ok_;	// we want to refuse negative standard deviations or zero amplitude
	}

//*************************
//***	    first		***
//*************************
// this is the function called by proFit before any series of calls to func

pascal void first (paramArray a,globalsPtrT InfPtr)
	{
// we do not need first. we will pass nil for it in 'main'
	}

//*************************
//***  freeUserMemory	***
//*************************
// this is the function called by proFit when your func is removed from the func menu

pascal void freeUserMemory (globalsPtrT InfPtr )
	{
		if (InfPtr->theHandle !=0) DisposHandle(InfPtr->theHandle);
// you only need to do this if you have allocated memory using the Handle InfPtr^.theHandle
// in main. If you didn't, don't implement this procedure and pass nil for it in 'main'
	}
	



//*************************
//***	    main		***
//*************************
// this is the function called by proFit when your func is first inserted in the func menu

pascal void  main 
									(str20 name,					//the name of the function }
									descrType descr,			// a description of the function for the parameters window }
									pascal void (**firstPtr)(paramArray, globalsPtrT),
														// pointer to the procedure 'first' or nil if not used }
									pascal char (**checkPtr)(short, paramArray, paramModesArray, paramNamesArray, globalsPtrT) ,
									 					// pointer to the procedure 'check' or nil if not used }
									pascal void (**funcPtr)(double*, paramArray, double*, globalsPtrT) ,
									 					// pointer to the procedure 'func' or nil if not used }
									pascal void (**derivativesPtr)(double*, paramArray, paramArray, globalsPtrT) ,
									 			// pointer to the procedure 'derivatives' or nil if not used }
									pascal void (**freeUserMPtr)(globalsPtrT) ,
									  			// pointer to the procedure 'FreeUserMemory' or nil if not used }
									short* nrParams,			// the number of parameters the function will use (  maxNrParams ) }
									paramArray a,			// the default values  for the parameters }
									paramModesArray mode,	// the default  modes for the parameters }
									paramNamesArray aName, 		// the parameter names }
									globalsPtrT InfPtr)				// pointer to a record containing global variables and some proFit globals available to the user }
/* The procedure main is called by proFit the first time the user code is linked. It sets pointers to }
{ all the routines that proFit will use and must do all the initializations that will be needed by 	}
{ routines. proFit does not perform any check on the values set by main. It will crash in a 		}
{ horrible and painful way if you do not do all the initializations properly. See the proFit user 	}
{ manual for more details. 																				}
*/
	
{ 

	InfPtr->theHandle = NewHandle(sizeof(MyGlobals));
	
/*this allocates space for the variables declared in the record 'MyGlobals' 
the function 'SizeOf' calculates the memory space that your record needs
just add new variable names to the record MyGlobals if you need more global variables*/

		if ((InfPtr->theHandle == nil) || (MemError() != noErr)) 
		SysBeep(1);
// handle error, if an error occured you are not allowed to use the Handle InfPtr^.theHandle}
		else
		{
			(**((HandToMyGlobals) InfPtr->theHandle)).a3=4;// not needed in this example
			(**((HandToMyGlobals) InfPtr->theHandle)).V1[10] = 10;// not needed in this example
// this lines are only here to show you what to do if you want to use global variables
// to avoid typing work, store ((HandToMyGlobals) InfPtr->theHandle) in a local variable
		};

		CopyToPascalStr("ErrorFunction", name,20); //name of our function

		// we store in the global array V[] the coefficients  needed by our function
				InfPtr->V[1] = 0.31938153;
				InfPtr->V[2] = -0.356563782;
				InfPtr->V[3] = 1.781477937;
				InfPtr->V[4] = -1.821255978;
				InfPtr->V[5] = 1.330274429;
	
		CopyToPascalStr("y := y0*ErrFunc(x-mean) + const, mean: mean value, SD: standard deviation", descr[0],80);
		CopyToPascalStr("The error function is defined as the integral over the gauss function.", descr[1],80);
		
		*firstPtr = 0; //if it were implemented, we would have written firstPtr = &first;}
		*checkPtr = &check;
		*funcPtr = &func;
		*freeUserMPtr = &freeUserMemory;
		*derivativesPtr = 0;//we didn't implement this function}
		*nrParams = 4;	//Number of parameters used. This MUST correspond to the number
					 	// of parameters you are using
		a[0] = 0;
		mode[0] = inactive;
		CopyToPascalStr("mean", aName[0],maxParamNameLength);
		a[1] = 1;
		mode[1] = inactive;
		CopyToPascalStr("SD", aName[1],maxParamNameLength);
		a[2] = 0;
		mode[2] = inactive;
		CopyToPascalStr("const", aName[2],maxParamNameLength);
		a[3] = 1;
		mode[3] = inactive;
		CopyToPascalStr("y0", aName[3],maxParamNameLength);


	}  // main }

