
#include "math.h"

#include "proFit_definitions.h"
/**************************  functions called by proFit *******************************/
/****** the definitions for the following functions must be provided by you  **********/

pascal void func (double* x, paramArray a, double* y,globalsPtrT InfPtr);

pascal void derivatives(double* x, paramArray a, paramArray dyda,globalsPtrT InfPtr);

pascal char check(short ParamNr, paramArray a, paramModesArray mode, paramNamesArray aName, globalsPtrT InfPtr);

pascal void first (paramArray a,globalsPtrT InfPtr);

pascal void freeUserMemory (globalsPtrT InfPtr );
	
pascal void  main (	str20 name,	//the name of the function }
					descrType descr,	// a description of the function for the parameters window
					pascal void (**firstPtr)(paramArray, globalsPtrT),
								// pointer to the procedure 'first' or nil if not used
					pascal char (**checkPtr)(short, paramArray, paramModesArray, paramNamesArray, globalsPtrT) ,
								// pointer to the procedure 'check' or nil if not used
					pascal void (**funcPtr)(double*, paramArray, double*, globalsPtrT) ,
					 			// pointer to the procedure 'func' or nil if not used
					pascal void (**derivativesPtr)(double*, paramArray, paramArray, globalsPtrT) ,
					 			// pointer to the procedure 'derivatives' or nil if not used }
					pascal void (**freeUserMPtr)(globalsPtrT) ,
					  			// pointer to the procedure 'FreeUserMemory' or nil if not used
					short* nrParams,			// the number of parameters the function will use (  maxNrParams ) }
					paramArray a,			// the default values  for the parameters
					paramModesArray mode,	// the default  modes for the parameters
					paramNamesArray aName, // the parameter names
					globalsPtrT InfPtr);	// pointer to a struct containing some proFit globals available to you
/***************************************************************************************/

typedef struct {		
				double  **outx,**outy;
				int oldindex;
				short pts,Npts;										
																
		} MyGlobals,**HandToMyGlobals;
/*
this works in the same way as usual variable declarations.The Handle
InfPtr->TheHandle must be initialized and set to point to this struct
*/

/*
To access comfortably the variables in this record we defined the type 'HandToMyGlobals'.
If H is of the type HandToMyGlobals and H == InfPtr->TheHandle we can access the variables
using (*H)->a3 etc. See the example in the procedure func.
*/



#include "mathematics.h"



static void Fillarray(const double* a1,double* a2, double yoffset, short npts, short index)
{
int i;for(i=1;i<=npts;i++) a2[i] = a1[i+index] - yoffset; 
}

static double extrapolate(double x, double x1, double y1, double x2, double y2)
{
return y1 + (y2 - y1) * (x - x1) / (x2 -x1);
}


//*************************
//***	    first		***
//*************************
// this is the function called by proFit before any series of calls to func

pascal void first (paramArray a,globalsPtrT InfPtr)
{
	double* subdatax,*subdatay,*xx,*yy;
	double average;
	short i;
	double b,sigb;
	short xCol=xColumn(InfPtr->proFitInfo), yCol=yColumn(InfPtr->proFitInfo);

	HandToMyGlobals H = (HandToMyGlobals) InfPtr->theHandle;
	(**H).pts=a[0];
	
	if ((**H).outx) DisposHandle((**H).outx);
	if ((**H).outy) DisposHandle((**H).outy);
	
	if ((xCol==0) || (yCol==0)) {SysBeep(0);InfPtr->proFitInfo->stopExec=1; return;}
	
	(**H).outx=(double**) NewHandle(sizeof(double) * (InfPtr->proFitInfo->nrRows + 1));
	(**H).outy=(double**) NewHandle(sizeof(double) * (InfPtr->proFitInfo->nrRows + 1));

	subdatax=(double*) NewPtr(sizeof(double) * (2 * (**H).pts + 1));
	subdatay=(double*) NewPtr(sizeof(double) * (2 * (**H).pts + 1));
	
	if ((subdatay!=0) &&(subdatax!=0) && ((**H).outx!=0) && ((**H).outy) && (MemErr==0)) {
		short count=1,index;
		double oldx,oldy;		
		HLock(H);
		HLock((**H).outx);HLock((**H).outy);
		xx=*((**H).outx);
		yy=*((**H).outy);
		for (i=1;i<=InfPtr->proFitInfo->nrRows;i++) {
			if (InfPtr->proFitInfo->testData(xCol,i) && InfPtr->proFitInfo->testData(yCol,i)) {
				xx[count]=InfPtr->proFitInfo->getData(xCol,i);
				yy[count]=InfPtr->proFitInfo->getData(yCol,i);
				count +=1;
			}
		}
		(**H).Npts = count - 1;

		if ((**H).Npts>1)	{
			sort2((**H).Npts,xx,yy);
			
			if ((**H).Npts<(**H).pts) (**H).pts=(**H).Npts;
			
			count=1;
			if ((**H).pts>2) 
			{	double siga,aa,chi2;
		 		fit(xx,yy,(**H).pts,0,0,&aa,&b,&siga,&sigb,&chi2);
				xx[count]= xx[1];			//first point
				yy[count]= aa + b * xx[1];
				count+=1;
				xx[count]= xx[(**H).pts];			//second point
				yy[count]= aa + b * xx[(**H).pts];
				index=(**H).pts;
				while (index < ((**H).Npts- 2* (**H).pts)) {
					Fillarray(xx,subdatax, xx[count], (**H).pts,index);
					Fillarray(yy,subdatay, yy[count], (**H).pts,index);
					linfit(subdatax,subdatay,(**H).pts,0,0,&aa,&siga,&chi2);
					
			//		fit(subdatax,subdatay,(**H).pts,0,0,&aa,&b,&siga,&sigb,&chi2);
			
					count += 1;index += (**H).pts;
				
					xx[count]= xx[index];			//store new point
					yy[count]= yy[count-1] + aa * subdatax[(**H).pts];
				}
				if ((**H).Npts-index>0) {
				// always true here: (**H).Npts-index > (**H).pts)	
					Fillarray(xx,subdatax, xx[count], (**H).Npts-index,index);
					Fillarray(yy,subdatay, yy[count], (**H).Npts-index,index);
					linfit(subdatax,subdatay,(**H).Npts-index,0,0,&aa,&siga,&chi2);
					count += 1;
					xx[count]= xx[(**H).Npts];			//store new point
					yy[count]= yy[count-1] + aa * subdatax[(**H).Npts-index];
				}
				(**H).Npts=count;
	
			} // if pts>2
		} // if Npts>1
		else  { SysBeep(1);InfPtr->proFitInfo->stopExec=1;}
		
		HUnlock(H);
		HUnlock((**H).outx);HUnlock((**H).outy);
		SetHandleSize((**H).outx,sizeof(double) * ((**H).Npts + 1));
		SetHandleSize((**H).outy,sizeof(double) * ((**H).Npts + 1));
	
		if (subdatax) DisposPtr(subdatax);
		if (subdatay) DisposPtr(subdatay);

	} else {SysBeep(1);(**H).Npts = 0;InfPtr->proFitInfo->stopExec=1;}
	(**H).oldindex=1;
}



//*************************
//***		func		***
//*************************
// this is the function used by proFit when fitting, plotting, etc .. 
void pascal func (double* x, paramArray a, double* y,globalsPtrT InfPtr)
{	HandToMyGlobals H = (HandToMyGlobals) InfPtr->theHandle;
	if ((**H).Npts>0) {
		short index=(**H).oldindex;
		double *xx,*yy;
	
		HLock(H);
		HLock((**H).outx);HLock((**H).outy);
		xx=*((**H).outx); yy=*((**H).outy);
		hunt(xx,(**H).Npts,*x,&index);
	
		if (index==0) { *y=extrapolate(*x,xx[1],yy[1],xx[2],yy[2]);return;}
		if (index==(**H).Npts) {*y=extrapolate(*x,xx[(**H).Npts-1],yy[(**H).Npts-1],xx[(**H).Npts],yy[(**H).Npts]);;return;}
	
		if ((**H).pts<=1) *y=yy[index] + (yy[index+1] - yy[index]) * (*x - xx[index])/(xx[index+1] - xx[index]);
		else
		{	double oldx1=xx[1],oldy1=yy[1],oldxN=xx[(**H).Npts],oldyN=yy[(**H).Npts];
			double pox,poy,p1x,p1y,p2x,p2y;
			xx[1]-=xx[2]-xx[1];yy[1]-=yy[2]-yy[1];
			xx[(**H).Npts]+=xx[(**H).Npts]-xx[(**H).Npts-1];yy[(**H).Npts]+=yy[(**H).Npts]-yy[(**H).Npts-1];
						
			if (*x < (xx[index]+xx[index+1])/2)
				{	pox=(xx[index-1]+xx[index])/2;poy=(yy[index-1]+yy[index])/2;
					p1x=xx[index];p1y=yy[index];
					p2x=(xx[index]+xx[index+1])/2;p2y=(yy[index]+yy[index+1])/2;
				}
			else
				{
					pox=(xx[index]+xx[index+1])/2;poy=(yy[index]+yy[index+1])/2;
					p1x=xx[index+1];p1y=yy[index+1];
					p2x=(xx[index+1]+xx[index+2])/2;p2y=(yy[index+1]+yy[index+2])/2;
				}
			Bezier(pox,poy,p1x,p1y,p2x,p2y,*x,y);
			xx[1]=oldx1;yy[1]=oldy1;
			xx[(**H).Npts]=oldxN;yy[(**H).Npts]=oldyN;
		}
		(**H).oldindex=index;
		HUnlock((**H).outx);HUnlock((**H).outy);
		HUnlock(H);
	}
}

//*************************
//***	 derivatives	***
//*************************
// this is the function used by proFit when fitting
 
pascal void derivatives(double* x, paramArray a, paramArray dyda,globalsPtrT InfPtr)
	{
//we pass nil for the derivatives procedure in main }
	}

//*************************
//***	    check		***
//*************************
// this is the function called by proFit when it changes one of the parameters

pascal char check(short ParamNr, paramArray a, paramModesArray mode, paramNamesArray aName, globalsPtrT InfPtr)
	{
		if ( ((ParamNr = 1) && (a[ParamNr-1] <= 0)) )
			return bad;
		else
			return ok_;
	}


//*************************
//***  freeUserMemory	***
//*************************
// this is the function called by proFit when your func is removed from the func menu

pascal void freeUserMemory (globalsPtrT InfPtr )
	{
		if (InfPtr->theHandle !=0) DisposHandle(InfPtr->theHandle);
	}
	



//*************************
//***	    main		***
//*************************
// this is the function called by proFit when your func is first inserted in the func menu

pascal void  main 
									(str20 name,					//the name of the function }
									descrType descr,			// a description of the function for the parameters window }
									pascal void (**firstPtr)(paramArray, globalsPtrT),
														// pointer to the procedure 'first' or nil if not used }
									pascal char (**checkPtr)(short, paramArray, paramModesArray, paramNamesArray, globalsPtrT) ,
									 					// pointer to the procedure 'check' or nil if not used }
									pascal void (**funcPtr)(double*, paramArray, double*, globalsPtrT) ,
									 					// pointer to the procedure 'func' or nil if not used }
									pascal void (**derivativesPtr)(double*, paramArray, paramArray, globalsPtrT) ,
									 			// pointer to the procedure 'derivatives' or nil if not used }
									pascal void (**freeUserMPtr)(globalsPtrT) ,
									  			// pointer to the procedure 'FreeUserMemory' or nil if not used }
									short* nrParams,			// the number of parameters the function will use (  maxNrParams ) }
									paramArray a,			// the default values  for the parameters }
									paramModesArray mode,	// the default  modes for the parameters }
									paramNamesArray aName, 		// the parameter names }
									globalsPtrT InfPtr)				// pointer to a record containing global variables and some proFit globals available to the user }
{ 
	InfPtr->theHandle = NewHandle(sizeof(MyGlobals));
/*this allocates space for the variables declared in the record 'MyGlobals' 
the function 'sizeof' calculates the memory space that your record needs
just add new variable names to the record MyGlobals if you need more global variables*/

		if ((InfPtr->theHandle == nil) || (MemError() != noErr)) 
		SysBeep(1);
// handle error, if an error occured you are not allowed to use the Handle InfPtr^.theHandle}
		else
		{ 	HandToMyGlobals H=(HandToMyGlobals) InfPtr->theHandle;
			(**H).outx=0;(**H).outy=0;
			(**H).pts=10;(**H).Npts=0;(**H).oldindex=1;
		};

		CopyToPascalStr("SmoothInterpolation", name,20); //name of our function

		
		CopyToPascalStr("y = interpolation of the data points in cols 1 and 2 by line pieces", descr[0],80);
		CopyToPascalStr("(#pts) is x-length of line pieces in data points. #pts=1 for lin. interpol", descr[1],80);

		*firstPtr = &first;
		*checkPtr = &check;
		*funcPtr = &func;
		*freeUserMPtr = &freeUserMemory;
		*derivativesPtr = 0;//we didn't implement this function}
		*nrParams = 1;	//Number of parameters used.		a[0] = 6;
		mode[0] = inactive;
		CopyToPascalStr("#pts", aName[0],maxParamNameLength);	
	}  // main }
	
