#include "math.h"

#include "mathematics.h"

static double sqr(double x) {return(x*x);}

void sort2(short n,double* ra,double* rb)
{
	short l,j,ir,i;
	double rrb,rra;

	l=(n >> 1)+1;
	ir=n;
	for (;;) {
		if (l > 1) {
			rra=ra[--l];
			rrb=rb[l];
		} else {
			rra=ra[ir];
			rrb=rb[ir];
			ra[ir]=ra[1];
			rb[ir]=rb[1];
			if (--ir == 1) {
				ra[1]=rra;
				rb[1]=rrb;
				return;
			}
		}
		i=l;
		j=l << 1;
		while (j <= ir)	{
			if (j < ir && ra[j] < ra[j+1]) ++j;
			if (rra < ra[j]) {
				ra[i]=ra[j];
				rb[i]=rb[j];
				j += (i=j);
			}
			else j=ir+1;
		}
		ra[i]=rra;
		rb[i]=rrb;
	}
}

void fit(	double x[],double y[],
			short ndata,
			double sig[],
			short mwt,
			double* a,
			double* b,
			double* siga,
			double* sigb,
			double* chi2)
{
	short i;
	double wt,t,sxoss,sx=0.0,sy=0.0,st2=0.0,ss,sigdat;
	double gammq();

	*b=0.0;
	if (mwt) { 
		ss=0.0;
		for (i=1;i<=ndata;i++) {
			wt=1.0/sqr(sig[i]);
			ss += wt;
			sx += x[i]*wt;
			sy += y[i]*wt;
		}
	} else {
		for (i=1;i<=ndata;i++) {
			sx += x[i];
			sy += y[i];
		}
		ss=ndata;
	}
	sxoss=sx/ss;
	if (mwt) {
		for (i=1;i<=ndata;i++) {
			t=(x[i]-sxoss)/sig[i];
			st2 += t*t;
			*b += t*y[i]/sig[i];
		}
	} else {
		for (i=1;i<=ndata;i++) {
			t=x[i]-sxoss;
			st2 += t*t;
			*b += t*y[i];
		}
	}
	*b /= st2;
	*a=(sy-sx*(*b))/ss;
	*siga=sqrt((1.0+sx*sx/(ss*st2))/ss);
	*sigb=sqrt(1.0/st2);
	*chi2=0.0;
	if (mwt == 0) {
		for (i=1;i<=ndata;i++)
			*chi2 += sqr(y[i]-(*a)-(*b)*x[i]);
		sigdat=sqrt((*chi2)/(ndata-2));
		*siga *= sigdat;
		*sigb *= sigdat;
	} else {
		for (i=1;i<=ndata;i++)
			*chi2 += sqr((y[i]-(*a)-(*b)*x[i])/sig[i]);
	}
}

 void linfit(	double x[],double y[],
			short ndata,
			double sig[],
			short mwt,
			double* a,
			double* siga,
			double* chi2)
{
	short i;
	double wt,t,sxoss,sxx=0.0,sxy=0.0,sumx=0,st2=0.0,ss;
	double gammq();

	*a=0.0;
	if (mwt) {
		ss=0.0;
		for (i=1;i<=ndata;i++) {
			wt=1.0/sqr(sig[i]);
			ss += wt;
			sumx += x[i];
			sxx += x[i]*x[i]*wt;
			sxy += y[i]*x[i]*wt;
		}
	} else {
		for (i=1;i<=ndata;i++) {
			sumx += x[i];
			sxx += x[i]*x[i];
			sxy += y[i]*x[i];
		}
		ss=ndata;
	}
	*a = sxy/sxx;
	*siga=sqrt(fabs(sumx/sxx));
	*chi2=0.0;
	if (mwt == 0) {
		for (i=1;i<=ndata;i++)
			*chi2 += sqr(y[i]-(*a)*x[i]);
		*siga *= sqrt((*chi2)/(ndata-2));
	} else {
		for (i=1;i<=ndata;i++)
			*chi2 += sqr((y[i]-(*a)*x[i])/sig[i]);
	}
}


void hunt(double* xx,short n,double x,short* jlo)
{
	short jm,jhi,inc,ascnd;

	ascnd=(xx[n] > xx[1]);
	if (*jlo <= 0 || *jlo > n) {
		*jlo=0;
		jhi=n+1;
	} else {
		inc=1;
		if (x >= xx[*jlo] == ascnd) {
			if (*jlo == n) return;
			jhi=(*jlo)+1;
			while (x >= xx[jhi] == ascnd) {
				*jlo=jhi;
				inc += inc;
				jhi=(*jlo)+inc;
				if (jhi > n) {
					jhi=n+1;
					break;
				}
			}
		} else {
			if (*jlo == 1) {
				*jlo=0;
				return;
			}
			jhi=(*jlo);
			*jlo -= 1;
			while (x < xx[*jlo] == ascnd) {
				jhi=(*jlo);
				inc += inc;
				*jlo=jhi-inc;
				if (*jlo < 1) {
					*jlo=0;
					break;
				}
			}
		}
	}
	while (jhi-(*jlo) != 1) {
		jm=(jhi+(*jlo)) >> 1;
		if (x > xx[jm] == ascnd)
			*jlo=jm;
		else
			jhi=jm;
	}
}


void Bezier(double pox,double poy,double p1x, double p1y,double p2x,double p2y,double x, double*y)
{
   double cx = 2.0 * (p1x - pox);
   double bx = p2x - 2.0 * p1x + pox;
   double cy = 2.0 * (p1y - poy);
   double by = p2y - 2.0 * p1y + poy;
   double t;
   if (fabs(bx)>0)	t= ( -cx + sqrt(fabs(sqr(cx) - 4*bx*(pox-x))) )/(2 * bx);
   else				t= (x - pox)/cx;
  	*y = (t * (cy + t * by)) + poy;
 }






