
/*
	Here follow the constant and type definitions of proFit.	
	You MUST use these declarations EXACTLY as they are given here
	ANY change in a type definition (e.g. changing the ranges of an array or the	
	order of the fields in a record) is guaranteed to cause very ugly consequences !!!
	For a detailed description of these declarations their use,
	refer to the proFit user manual.
*/

	
	#define maxNrParams  30			// the maximum number of parameters 
	#define	maxParamNameLength  9	// the maximum length of a parameter name 
	#define	maxNrInputValues   6	// the maxiumum number of input values in the input dialog box

	
// the following group of type definitions is only needed for functions
		
	typedef unsigned char str80[82];			// this string must not be larger than 80!!

	typedef str80 descrType[2];
		
	typedef enum {update, ok_, bad} checkPAnswer;	//the possible return values of the 'check' function 

	typedef double paramArray[maxNrParams];		//30=maxNrParams, the numerical values of the parameters 

	typedef enum {active, inactive, constant} modeType;	// the dynamic state of a parameter 
	
	typedef char paramModesArray[maxNrParams];	// the states of all the parameters 
												// only assign modeTypes to this array
												// char is only needed to make the elements
												// one Byte long.

	typedef unsigned char paramNameT[maxParamNameLength+1];	//maxParamNameLength = 9, the name of a prameter for the parameters window 
			
	typedef paramNameT paramNamesArray[maxNrParams];// the names of all the parameters 


// this second group of type definitions is used for functions and programs

	typedef double *extPtr;

	//the following record is used as a parameter block to several routines provided by proFit
	//now or in future versions
	
	typedef struct  {
			short result;		// noErr or error code 
			short what;			/*	 the operation: 
									 1:	newCurve 
									 2:	moveto
									 3:	lineto 
									 4:	getGrafCoordinates 
									11:	random
						   20,21,22,23: xColumn,yColumn,xErrColumn,yErrColumn 		*/
			union {
					struct	//newcurve
					{	unsigned char* curveName;		//name of the curve
						double color, dash, thick;		//line type
					} s1;
					
					struct {double y, x;} s2,s3;		//lineto, moveto
							
					struct {extPtr	ymax, ymin, xmax, xmin;} s4;	//getGrafCoordinates
						
					double	retVal;		//random, xColumn,yColumn,xErrColumn,yErrColumn
				} u;

		} runTimeParamRec;

	typedef unsigned char str20[21];		// the name of the function or program 

	typedef struct {double* x; unsigned char* s;} inputRec[maxNrInputValues];	// information type for the input dialog box

	// the following struct contains different infos provided by proFit 
	typedef struct	{	
			short nrRows, nrCols;	// the number of rows and columns in the actual data window }
			char stopExec;		// if set to 1, proFit will stop program execution as soon as returned to proFit }
		
	// the following are pointers to procedures and functions provided by proFit
	// Str255 is defined as unsigned char[256]
				pascal void		(*write) (Str255);
				pascal void		(*writeln) (Str255);
				pascal double	(*getData) (short Column,short Row);
				pascal void  	(*setData) (short,short,double);
				pascal char		(*testData) (short,short);
				pascal void		(*clearData) (short,short);
				pascal char		(*testStop) (void);
				pascal Rect 	(*getSelection) (void);
				pascal void		(*setColName) (short,unsigned char[256]);
				pascal char		(*inputBox) (short, inputRec);
				pascal void		(*alert) (unsigned char[256]);
				pascal char		(*ask) (short*,unsigned char[256]);
				pascal void 	(*rtProc) (runTimeParamRec*);
				
			} proFitInfoRecT;
			
	typedef proFitInfoRecT* infoPtrT;

	//the following struct is passed to all the functions that are called by proFit
	//it contains a global array and a Handle that you can use freely
	typedef struct	{	
			infoPtrT proFitInfo;		// the information explained above 
			double V[31];				// some easy to reference double values 
			Handle theHandle;			// Handle to an individual block of memory
			} userGlobalsRecordT, *globalsPtrT;




/**************************  other functions provided by proFIt ************************/
/** the code for the following fucnctions is  found in the file proFit_definitions.c ***/
	
	void newCurve (double thick,short dash, short color,char* name, infoPtrT iP);

	void grMoveto (double x, double y, infoPtrT iP);

	void grLineto (double x, double y, infoPtrT iP);

	void getGrafCoordinates (double* xmin, double* xmax, double* ymin, double* ymax,infoPtrT iP);

	double pRandom (infoPtrT iP);

	short xColumn (infoPtrT iP);

	short yColumn (infoPtrT iP);

	short xErrColumn (infoPtrT iP);

	short yErrColumn (infoPtrT iP);
/***************************************************************************************/





/******    a small utility function, defined in proFit_definitions.c   ****************/

void CopyToPascalStr(char* s1,Str255 s2, short limit);
// utility function to produce the pascal type strings needed by proFit
// that is, with the first char containing the length and not zero-terminated

