
 #include "proFit_definitions.h" 

/**************************  functions called by proFit *******************************/
/****** the definitions for the following functions must be provided by you  **********/

pascal void run(globalsPtrT InfPtr);
// this is the routine that does the real work for you ... 
// In this example it just fills the current data window with a multiplication table 

pascal void main (	str20 name,			// the name of the function 
					pascal void (**runPtr) (globalsPtrT),			// pointer to the procedure 'run' or nil if not used }
					pascal void (**freeUserMPtr) (globalsPtrT),		// pointer to the procedure 'freeUserMemory' or nil if not used }
					globalsPtrT InfPtr);		// pointer to a record containing global variables and some proFit globals available to the user }
/***************************************************************************************/
/*
 You MUST use the following declaration of the procedure main EXACTLY as it is given here. 	
 ANY change is guaranteed to cause very ugly crashes !!!	

 The procedure main is called by proFit the first time the user code is linked. It sets pointers to 
 all the routines that proFit will use and must do all the initializations that will be needed by 		
 routines. proFit does not perform any check on the values set by main. It will crash in a 		
 horrible and painful way if you do not do all the initializations properly. See the proFit user 	
 manual for more details. 																				
*/

/*
 This is a sample program which can be compiled in any compiler and linked to proFit.		
 Proceed as follows: 
    compile this file in your compiler  tell your compiler to save it as a CODE resource. 
    (Most current compilers on the Macintosh (MPW, Think C, ..) 	
    allow to make a code resource  this feature is used for the compilation of device drivers,
    desk accessories, window definition procedures etc. Find more details in your compiler's 	
    manual.) The code resource must be of type 'pgrf' or 'pgrn' depending on wether you generate 
	code for the floating point unit (math coprocessor) or not !, its ID is not important but must
	be larger than 128. 	
    Save this resource into a file and add it to proFit using the Link... commmand. 
 IMPORTANT NOTE: 
 The special way in which global variables are treated in Macintosh compilers makes it 	
 impossible to use global variables in spearated code segments. This means that you  	
 MUST NOT define any variables outside your procedures!		
 (You could store global data in the userglobals record.) 	
*/	

#include <math.h>
	
		typedef double data[8191];
		typedef data* dataptr;

/*********************************************************************************************/
	static void WritelnString (Str255 s, globalsPtrT InfPtr)
	{			InfPtr->proFitInfo->writeln(s);
	}

static double sqr(double x) {return (x*x);}

static void fft (dataptr x, int num,char inverse,globalsPtrT InfPtr)
	{
			int ii, jj, n, mmax, m, j, istep, i;
			double wtemp, wr, wpr, wpi, wi, theta, tempr, tempi;
	
		n = num;
		j = 1;
		for (ii = 1;ii<= (n / 2);ii++) 
			{
				i = 2 * ii - 1;
				if (j > i) 
					{
						tempr = (*x)[j];
						tempi = (*x)[j + 1];
						(*x)[j] = (*x)[i];
						(*x)[j + 1] = (*x)[i + 1];
						(*x)[i] = tempr;
						(*x)[i + 1] = tempi;
					}
				m = n / 2;
				while ((m >= 2) && (j > m)) 
					{
						j = j - m;
						m = m / 2;
					}
				j = j + m;
				if (InfPtr->proFitInfo->testStop()) 		// test if proFit or a user tries to stop the program }
					goto end;
			}		//for ii = 1 to (n div 2) do }

		mmax = 2;
		while (n > mmax) 
			{
				istep = 2 * mmax;
				theta = 6.283185307959 / mmax;
				if (inverse) 
					theta = -theta;
				wpr = -2 * sqr(sin(0.5 * theta));
				wpi = sin(theta);
				wr = 1;
				wi = 0;
				for (ii = 1;ii<= (mmax / 2);ii++) 
					{
						m = 2 * ii - 1;
						for (jj = 0;jj<= ((n - m) / istep);jj++) 
							{
								i = m + jj * istep;
								j = i + mmax;
								tempr = wr * (*x)[j] - wi * (*x)[j + 1];
								tempi = wr * (*x)[j + 1] + wi * (*x)[j];
								(*x)[j] = (*x)[i] - tempr;
								(*x)[j + 1] = (*x)[i + 1] - tempi;
								(*x)[i] = (*x)[i] + tempr;
								(*x)[i + 1] = (*x)[i + 1] + tempi;
							}		// for jj := 0 to ((n - m) div istep) do }
						wtemp = wr;
						wr = wr + wr * wpr - wi * wpi;
						wi = wi + wi * wpr + wtemp * wpi;
						if (InfPtr->proFitInfo->testStop()) 	//test if proFit or a user tries to stop the program }
							goto end;
					}	// for ii := 1 to (mmax div 2) do }
				mmax = istep;
			}		// while n>mmax }
			end:;
	}		// fft }

/***************************************************************************************/
static	void shiftFFT (dataptr x, int np)
	{
		double xx;
		int i, np2;
		for (i = 1;i<= np;i++) 
			{
				xx = (*x)[i];
				(*x)[i] = (*x)[i + np];
				(*x)[i + np] = xx;
			}
	}	// shiftFFT }

/***************************************************************************************/
static void CleanUpRun(dataptr x, globalsPtrT InfPtr)
{	InfPtr->proFitInfo->stopExec = 1;		// proFit will do the necessary things ... }
	if (x!=0)  {SysBeep(1);DisposePtr(x);x=0;}
}

pascal void run(globalsPtrT InfPtr)
// this is the routine that does the real work for you ... 
// it performs a fourier transform on a column in the actual data window 

	{
			inputRec r;
			double	ex1=InfPtr->V[1],
					ex2=InfPtr->V[2],
					ex3=InfPtr->V[3],
					ex4=InfPtr->V[4],
					ex5=InfPtr->V[5],
					x0, xinc, y0, yinc, yoff;
			int i, xCol, yCol;
			dataptr x;		// pointer to the data array }
			int nrData;		// the number of data (integer power of 2) }
			char inverse;	// false if normal FFT should be done, true for reverse FFT }
			short answer;
		
		if (InfPtr->V[0] == 0)
			{		// this is the first time this program runs }
				InfPtr->V[0] = 1;
			//\p instructs the compiler to produce a pascal string
				if (InfPtr->proFitInfo->ask(&answer,"\pDo you want to get some information on this program in the Results window ?"))
					goto end;				// stop button pressed 
				else 
				  {if (answer == 1) 	//yes button pressed }
					{
						WritelnString("\pThe number of data points must be between 4 and  4096.",InfPtr);
						WritelnString("\pThe program sets the number of data points to the next smaller power of 2",InfPtr);
						WritelnString("\pFirst column No of data means the column number where the time values are stored.",InfPtr);
						WritelnString("\pThe real and imaginary values of your data must be in the succeding two columns.",InfPtr);
						WritelnString("\pFirst column No of FFT means the column number where the frequency values will be stored.",InfPtr);
						WritelnString("\pThe real and imaginary part of the Fourier transformed data will be in the succeding two columns.",InfPtr);
						WritelnString("\pOffset of FFT defines an additive constant for the frequency values.",InfPtr);
						goto end;
					}
				  }
			}

		x = (dataptr) NewPtr(sizeof(data));			// prepare memory for data }
		if (x == 0) 
			{CleanUpRun(x,InfPtr);goto end;}


		r[0].x =&ex1;
		r[0].s = "\pNumber of data points";
		r[1].x = &ex2;
		r[1].s = "\pFirst column No. of data";
		r[2].x = &ex3;
		r[2].s = "\pFirst column No of FFT";
		r[3].x = &ex4;
		r[3].s = "\p    Offset of FFT";
		r[4].x = &ex5;
		r[4].s = "\pFFT (>0) ; inverse FFT (<0) ?";
		if (InfPtr->proFitInfo->inputBox(5, r))
			{CleanUpRun(x,InfPtr);goto end;}	// if something is not ok we quit 
		else
			{	// if everything is ok we do something 
				ex1 = fabs(ex1);
				if ((ex1 > 4096) || (ex1 < 4))
					{CleanUpRun(x,InfPtr);goto end;}
				nrData = exp(log(2) * floor(log(ex1) / log(2)));
				ex2 = fabs(ex2);
				if ((ex2 > 97) || (ex2 < 1)) 
					{CleanUpRun(x,InfPtr);goto end;}
				xCol = (ex2);
				ex3 = fabs(ex3);
				if ((ex3 > 97) || (ex3 < 1)) 
					{CleanUpRun(x,InfPtr);goto end;}
				yCol = (ex3);
				yoff = ex4;
				if (ex5 >= 0) 
					inverse = 0;
				else
					inverse = 1;

				if (InfPtr->proFitInfo->testData(xCol, 1)) 
					{
						x0 = InfPtr->proFitInfo->getData(xCol, 1);
						if (InfPtr->proFitInfo->testData(xCol, 2))
							xinc =InfPtr->proFitInfo->getData(xCol, 2) - x0;
					}

				
				for (i = 1;i<= nrData;i++)			// read data from window 
					{
						if (InfPtr->proFitInfo->testData(xCol + 1, i))	//test if data value is ok }
							(*x)[2 * i - 1] = InfPtr->proFitInfo->getData(xCol + 1, i);
						else
							(*x)[2 * i - 1] = 0;
						if (InfPtr->proFitInfo->testData(xCol + 2, i))
							(*x)[2 * i] = InfPtr->proFitInfo->getData(xCol + 2, i);
						else
							(*x)[2 * i] = 0;
						if (InfPtr->proFitInfo->testStop())
							{CleanUpRun(x,InfPtr);goto end;}
					}
				
				fft(x, 2 * nrData, inverse, InfPtr);	// fft calculations }
				yinc = 1 / (xinc*nrData) ;
				if (!inverse) 
					{
						shiftFFT(x, nrData);
						y0 = yoff - 1 / xinc / 2;
					}
				else
					{
						y0 = yoff;
						for (i = 1;i<= nrData / 2;i++) 
							{
								(*x)[4 * i - 3] = (*x)[4 * i - 3] / nrData;
								(*x)[4 * i - 2] = (*x)[4 * i - 2] / nrData;
								(*x)[4 * i - 1] = -(*x)[4 * i - 1] / nrData;
								(*x)[4 * i] = -(*x)[4 * i] / nrData;
							}
					}
				if (InfPtr->proFitInfo->testStop() )
					{CleanUpRun(x,InfPtr);goto end;}
				InfPtr->proFitInfo->setColName(yCol,"\pfrequency");
				InfPtr->proFitInfo->setColName(yCol + 1,"\pFFT real");
				InfPtr->proFitInfo->setColName(yCol + 2, "\pFFT imag.");
				for (i = 1; i<= nrData;i++)			// output of data into window }
					{
						InfPtr->proFitInfo->setData(yCol, i, y0 + (i - 1.0) * yinc);
						InfPtr->proFitInfo->setData(yCol + 1, i, (*x)[2 * i - 1]);
						InfPtr->proFitInfo->setData(yCol + 2, i, (*x)[2 * i]);
						if (InfPtr->proFitInfo->testStop() ) 
							{CleanUpRun(x,InfPtr);goto end;}
					}

				if (x != 0) 
					DisposPtr(x);		// we don't need the memory any more }
			}
			end:
		InfPtr->V[1] = ex1;	//these are the default values for the dialog box asking for dataNr, ColumnNrs 
		InfPtr->V[2] = ex2;
		InfPtr->V[3] = ex3;
		InfPtr->V[4] = ex4;
		InfPtr->V[5] = ex5;
;
	}

/***************************************************************************************/

pascal void main (	str20 name,			// the name of the function 
					pascal void (**runPtr) (globalsPtrT),			// pointer to the procedure 'run' or nil if not used }
					pascal void (**freeUserMPtr) (globalsPtrT),		// pointer to the procedure 'freeUserMemory' or nil if not used }
					globalsPtrT InfPtr)	// pointer to a record containing global variables and some proFit globals available to the user }
	{	
		CopyToPascalStr("Complex FFT", name, 20);
		*runPtr = &run;
		InfPtr->V[0] = 0;	//use it as boolean value to test for the first time this program runs }
		InfPtr->V[1] = 8;	//these are the default values for the dialog box asking for dataNr, ColumnNrs 
		InfPtr->V[2] = 1;
		InfPtr->V[3] = 4;
		InfPtr->V[4] = 0;
		InfPtr->V[5] = 1;
		*freeUserMPtr = 0;	//we didn't use any global data in a handle ! }
	} 

/***************************************************************************************/