 #include "proFit_definitions.h" 

/**************************  functions called by proFit *******************************/
/****** the definitions for the following functions must be provided by you  **********/

pascal void run(globalsPtrT InfPtr);
// this is the routine that does the real work for you ... 
// In this example it just fills the current data window with a multiplication table 

pascal void main (	str20 name,			// the name of the function 
					pascal void (**runPtr) (globalsPtrT),			// pointer to the procedure 'run' or nil if not used }
					pascal void (**freeUserMPtr) (globalsPtrT),		// pointer to the procedure 'freeUserMemory' or nil if not used }
					globalsPtrT InfPtr);	// pointer to a record containing global variables and some proFit globals available to the user }
/***************************************************************************************/
/*
 You MUST use the following declaration of the procedure main EXACTLY as it is given here. 	
 ANY change is guaranteed to cause very ugly crashes !!!	

 The procedure main is called by proFit the first time the user code is linked. It sets pointers to 
 all the routines that proFit will use and must do all the initializations that will be needed by 		
 routines. proFit does not perform any check on the values set by main. It will crash in a 		
 horrible and painful way if you do not do all the initializations properly. See the proFit user 	
 manual for more details. 																				
*/

/*
 This is a sample program which can be compiled in any compiler and linked to proFit.		
 Proceed as follows: 
    compile this file in your compiler  tell your compiler to save it as a CODE resource. 
    (Most current compilers on the Macintosh (MPW, Think C, ..) 	
    allow to make a code resource  this feature is used for the compilation of device drivers,
    desk accessories, window definition procedures etc. Find more details in your compiler's 	
    manual.) The code resource must be of type 'pgrf' or 'pgrn' depending on wether you generate 
	code for the floating point unit (math coprocessor) or not !, its ID is not important but must
	be larger than 128. 	
    Save this resource into a file and add it to proFit using the Link... commmand. 
 IMPORTANT NOTE: 
 The special way in which global variables are treated in Macintosh compilers makes it 	
 impossible to use global variables in spearated code segments. This means that you  	
 MUST NOT define any variables outside your procedures!		
 (You could store global data in the userglobals record.) 	
*/	

 #include <math.h> //for cos and sin

pascal void run(globalsPtrT InfPtr)
// this is the routine that does the real work for you ... 
	{
			double	ex1=InfPtr->V[1], ex2=InfPtr->V[2],	/* buffers for input */
					ex3=InfPtr->V[3], ex4=InfPtr->V[4];	/* default values are stored in V[..] */
			inputRec r;					/* dto */
			int i;
			double angle,buf;

// get the parameters from user, for this we prepare the inputRec r */
		r[0].x = &ex1;
		r[0].s = "\pFrequency x";

		r[1].x = &ex2;
		r[1].s = "\pFrequency y";

		r[2].x = &ex3;
		r[2].s = "\pStep [rad]";

		r[3].x = &ex4;
		r[3].s = "\pNumber of points";
		if (InfPtr->proFitInfo->inputBox(4, r))
			goto end;

		if (ex4 > 32767) 			/* make sure ex4 can be cast to an integer */
			ex4 = 32767;
		else if (ex4 < 0)
			ex4 = 0;
		InfPtr->V[1] = ex1;				/* defaults for next time */
		InfPtr->V[2] = ex2;
		InfPtr->V[3] = ex3;
		InfPtr->V[4] = ex4;

	/* start drawing the curve: */
		newCurve(0.5, 1, 1, "Lissajous", InfPtr->proFitInfo);	/* thickness 0.5, no dash, black */
		grMoveto(1, 0, InfPtr->proFitInfo);						/* move to the start point */
		angle = 0;
		for (i = 1;i<= ex4;i++)
			{	grLineto(cos(ex1 * angle), sin(ex2 * angle), InfPtr->proFitInfo);
				angle = angle + ex3;
			} /* for i */ 
		end:;
	} /* run */

/*--------------------------------------------------------------------------------------------------------*/

pascal void main (	str20 name,			// the name of the function 
					pascal void (**runPtr) (globalsPtrT),			// pointer to the procedure 'run' or nil if not used }
					pascal void (**freeUserMPtr) (globalsPtrT),		// pointer to the procedure 'freeUserMemory' or nil if not used }
					globalsPtrT InfPtr)	// pointer to a record containing global variables and some proFit globals available to the user }
	{
		CopyToPascalStr("Lissajous",name,20);
		*runPtr = &run;
		*freeUserMPtr = 0;		/* not used */

	/* initialize default values: */
		InfPtr->V[1] = 3;			/* the frequency along x */
		InfPtr->V[2] = 4;			/* the frequency along y */
		InfPtr->V[3] = 0.5;			/* the step width */
		InfPtr->V[4] = 300;			/* the number of points */

	}  /* main */

/*--------------------------------------------------------------------------------------------------------*/
