unit lissajous;

{ This unit gives an example of an external module to proFit. It uses the routines }
{ newCurve, grMoveto, and grLineto that were added to proFit 4.1 to access the current graph. }
{ To use this module you must compile it, link its code to proFit, open a graf in profit }
{ with x and y-axes from -1 to 1. Then run the user program 'Lissajous' and see what happens }

{--------------------------------------------------------------------------------------------------------}

interface

	uses
		proFitDefinitions;

{ You MUST use the following declaration of the procedure main EXACTLY as it is given here. 	}
{ ANY change is guaranteed to cause very ugly crashes !!!											}

{ The procedure main is called by proFit the first time the user code is linked. It sets pointers to }
{ all the routines that proFit will use and must do all the initializations that will be needed by 		}
{ routines. proFit does not perform any check on the values set by main. It will crash in a 		}
{ horrible and painful way if you do not do all the initializations properly. See the proFit user 	}
{ manual for more details. 																				}


	procedure main (var name: str20;			{ the name of the function }
									var runPtr: ptr;				{ pointer to the procedure 'run' or nil if not used }
									var freeUserMPtr: ptr; 		{ pointer to the procedure 'freeUserMemory' or nil if not used }
									InfPtr: GlobalsPtrT);			{ pointer to a record containing global variables and some proFit globals available to the user }

{--------------------------------------------------------------------------------------------------------}

implementation

{ This is a sample program which can be compiled in any compiler and linked to proFit.				}
{ Proceed as follows: }
{    compile this unit in your compiler  tell your compiler to save it as a CODE resource. 		}
{    (Most current compilers on the Macintosh (MPW, ThinkPascal, Lightspeed Pascal ..) 			}
{    allow to make a code resource  this feature is used for the compilation of device drivers, 	}
{    desk accessories, window definition procedures etc. Find more details in your compiler's 	}
{    manual.) The code resource must be of type 'pgrf' or 'pgrn' depending on wether you generate }
{	code for the floating point coprocessor or not !, its ID is not important. 						}
{    Save this resource into a file and add it to proFit using the Link... commmand. 					}
{ IMPORTANT NOTE: }
{ The special way in which global variables are treated in Macintosh compilers makes it 			}
{ impossible to use global variables in spearated code segments. This means that you  			}
{ MUST NOT define any varibales outside your procedures!											}
{ (You could store global data in the userglobals record.) 											}


{--------------------------------------------------------------------------------------------------------}

	procedure run (InfPtr: globalsPtrT);
{ this is the routine that does the real work for you ... }
{ it make a fourier transform on a column in the actual data window }

		var
			s1, s2, s3, s4: str255;			{ buffers for input }
			ex1, ex2, ex3, ex4: extended;	{ dto }
			r: inputRec;							{ dto }
			i: integer;
			angle: extended;

	begin
	{ get the parameters from user, for this we prepare the inputRec r }
		ex1 := infPtr^.v[1];				{ default values are stored in v[..] }
		s1 := 'Frequency x';
		r[1].x := pointer(@ex1);
		r[1].s := pointer(@s1);
		ex2 := infPtr^.v[2];
		s2 := 'Frequency y';
		r[2].x := pointer(@ex2);
		r[2].s := pointer(@s2);
		ex3 := infPtr^.v[3];
		s3 := 'Step [rad]';
		r[3].x := pointer(@ex3);
		r[3].s := pointer(@s3);
		ex4 := infPtr^.v[4];
		s4 := 'Number of points';
		r[4].x := pointer(@ex4);
		r[4].s := pointer(@s4);
		if InputBox(4, r, InfPtr^.proFitInfo^.inputPtr) then
			exit(run);

		if ex4 > maxint then			{ make sure ex4 can be cast to an integer }
			ex4 := maxint
		else if ex4 < 0 then
			ex4 := 0;
		infPtr^.v[1] := ex1;				{ defaults for next time }
		infPtr^.v[2] := ex2;
		infPtr^.v[3] := ex3;
		infPtr^.v[4] := ex4;

	{ start drawing the curve: }
		newCurve(0.5, 1, 1, 'Lissajous', infPtr^.profitInfo);	{ thickness 0.5, no dash, black }
		grMoveto(1, 0, infPtr^.profitInfo);						{ move to the start point }
		angle := 0;
		for i := 1 to round(ex4) do
			begin
				grLineto(cos(ex1 * angle), sin(ex2 * angle), infPtr^.profitInfo);
				angle := angle + ex3;
			end; { for i }

	end; { run }

{--------------------------------------------------------------------------------------------------------}

	procedure main (var name: str20;			{ the name of the function }
									var runPtr: ptr;				{ pointer to the procedure 'run' or nil if not used }
									var freeUserMPtr: ptr; 		{ pointer to the procedure 'freeUserMemory' or nil if not used }
									InfPtr: globalsPtrT);			{ pointer to a record containing global variables and some proFit globals available to the user }
	begin
		name := 'Lissajous';
		runPtr := @Run;
		freeUserMPtr := nil;		{ not used }

	{ initialize default values: }
		InfPtr^.V[1] := 3;			{ the frequency along x }
		InfPtr^.V[2] := 4;			{ the frequency along y }
		InfPtr^.V[3] := 0.5;			{ the step width }
		InfPtr^.V[4] := 500;			{ the number of points }

	end;  { main }

{--------------------------------------------------------------------------------------------------------}


end.